import MuseScore 3.0
import QtQuick 2.0

MuseScore {

    version: "1.0";
    description: "Force all chord symbols to chosen staff\n"
    menuPath: "Plugins.AlignedHarmonies"

        id: replaceChordSyms

        Component.onCompleted: {
            if (mscoreMajorVersion >= 4) {
                replaceChordSyms.title = "Force all chord symbols to chosen staff"
            }
        }


    function removeHarmony(cursor) {
        var segment = cursor.segment
        var aCount = 0;
        var annotation = segment.annotations[aCount];
        while (annotation) {
            if (annotation.type == Element.HARMONY)
                removeElement(annotation);
            annotation = segment.annotations[++aCount];
        }
    }

    function setCursorToTime(cursor, time) {
        cursor.rewind(0);
        while (cursor.segment) {
            var current_time = cursor.tick;
            if (current_time >= time) {
                return true;
            }
            cursor.next();
        }
        cursor.rewind(0);
        return false;
    }

    function collectchordnames(segment, chordnames, ticks, cursor, endTick) {
        while (segment && segment.tick < endTick) {
            var aCount = 0;
            var annotation = segment.annotations[aCount];

            while (annotation) {
                if (annotation.type == Element.HARMONY) {
                    var harmony = annotation;
                    var str = harmony.text;
                    if (harmony.text != "") {
                        console.log("harmony tick= ", harmony.text + " " + segment.tick)
                        chordnames.push(harmony.text);
                        ticks.push(segment.tick);
                        // remove the chord symbol at original location
                        removeElement(annotation);
                    }
                    annotation = segment.annotations[++aCount];
                }
                //console.log("cursor.tick = ", cursor.tick)
                //console.log("segment.tick = ", segment.tick)

            }
            segment = segment.next;
        }
    }

    function smartQuit() {
        if (mscoreMajorVersion < 4) {
            Qt.quit()
        } else {
            quit()
        }
    } //smartQuit

    function findHarmony(chordnames, ticks, tick) {
        for (var i in ticks) {
            if (ticks[i] == tick) {
                // mark as found so that it only gets used once
                ticks[i] = -1;
                return chordnames[i];
            }
        }
        return "";
    }

    onRun: {
        var targetStaff = 1; // first staff = 1
        console.log("targetStaff = ", targetStaff);
        if (typeof curScore === 'undefined') {
            Qt.quit();
        }

        var cursor = curScore.newCursor(),
            startStaff,
            endStaff,
            endTick,
            fullScore = false;

        cursor.rewind(1);


        if (!cursor.segment) { // no selection
            fullScore = true;
            startStaff = 0; // start with 1st staff
            endStaff = curScore.nstaves - 1; // and end with last
            endTick = curScore.lastSegment.tick + 1;
        } else {
            startStaff = cursor.staffIdx;
            cursor.rewind(2);
            if (cursor.tick === 0) {
                // this happens when the selection includes
                // the last measure of the score.
                // rewind(2) goes behind the last segment (where
                // there's none) and sets tick=0
                endTick = curScore.lastSegment.tick + 1;
            } else {
                endTick = cursor.tick;
            }
            endStaff = cursor.staffIdx;
        }

        cursor.rewind(1); // beginning of selection
        if (fullScore) { // no selection
            cursor.rewind(0); // beginning of score
        }
        cursor.voice = 0;


        // collect the existing chord symbols and their segment.tick
        var chordnames = [];
        var ticks = [];
        var segment = curScore.firstSegment();
        collectchordnames(segment, chordnames, ticks, cursor, endTick);

        var cc = 0;
        var nc = 0;
        
        // walk thru the selection with the cursor
        // if a chord symbol exists at cursor.tick put the chord symbol there
        for (var staff = startStaff; staff < endStaff + 1; staff++) {
            cursor.rewind(1); // beginning of selection
            cursor.staffIdx = staff;
            if (fullScore) // no selection
                cursor.rewind(0); // beginning of score

            while (cursor.segment && (fullScore || cursor.tick < endTick)) {
                var harmony = findHarmony(chordnames, ticks, cursor.tick)
                if (harmony != "") {
                    // there may be no segment in the track of choice at this tick
                    // so use the new cursor trick
                    var newharmony = newElement(Element.HARMONY)
                    newharmony.text = harmony;
                    var c = curScore.newCursor() // get a new cursor
                    c.track = cursor.track // set it to the current track & tick
                    c.rewindToTick(cursor.tick) // and the current tick
                    c.track = (targetStaff - 1) * 4 // now switch it to target track
                    c.add(newharmony)
                }
                cursor.next();
            }
        }
        smartQuit();
    } // end onRun
}